@@ -48,7 +48,11 @@
 #include "nsIDocShell.h"
 #include "nsIObserverService.h"
 #include "nsIAppStartup.h"
+#ifdef MOZILLA_1_9_2_BRANCH
 #include "nsXPFEComponentsCID.h"
+#else
+#include "nsToolkitCompsCID.h"
+#endif
 #include "nsISupportsPrimitives.h"
 #include "nsIAppShellService.h"
 #include "nsAppShellCID.h"
@@ -726,7 +730,7 @@ NS_IMETHODIMP nsMsgShutdownService::Observe(nsISupports *aSubject,
       // If the attempted quit was a restart, be sure to restart the app once
       // the tasks have been run. This is usually the case when addons or
       // updates are going to be installed.
-      if (nsDependentString(aData).EqualsLiteral("restart"))
+      if (aData && nsDependentString(aData).EqualsLiteral("restart"))
         mQuitMode |= nsIAppStartup::eRestart;
     }
 
