@@ -128,8 +128,8 @@ private:
 public:
   nsCOMPtr <nsIURI> mURL;
   nsCOMPtr <nsILocalFile>        mTmpFile;         // The temp file to which we save it 
-  nsCOMPtr<nsIFileOutputStream>  mOutFile;          
-  nsIRequest            *mRequest;          // The live request used while fetching an attachment
+  nsCOMPtr<nsIOutputStream>  mOutFile;          
+  nsCOMPtr<nsIRequest> mRequest; // The live request used while fetching an attachment
   nsMsgCompFields       *mCompFields;       // Message composition fields for the sender
   PRBool                m_bogus_attachment; // This is to catch problem children...
   
