@@ -50,7 +50,6 @@ function imapDaemon(flags, syncFunc) {
   this.inbox = new imapMailbox("INBOX", null, this.uidvalidity++);
   this.root.addMailbox(this.inbox);
   this.namespaces.push(this.root);
-
   this.syncFunc = syncFunc;
 }
 imapDaemon.prototype = {
@@ -95,7 +94,7 @@ imapDaemon.prototype = {
       }
       if (!mailbox)
         return null;
-      
+
       // Now we continue like normal
       var names = name.split(mailbox.delimiter);
       names.splice(0, 1);
@@ -139,7 +138,7 @@ imapDaemon.prototype = {
       // Only delete now so we don't screw ourselves up if creation fails
       this.deleteMailbox(oldBox);
       oldBox._parent = box == this.root ? null : box;
-      let newBox = new imapMailbox(oldBox.name, box, this.uidvalidity++);
+      let newBox = new imapMailbox(subName, box, this.uidvalidity++);
       newBox._messages = oldBox._messages;
       box.addMailbox(newBox);
 
@@ -336,14 +335,14 @@ imapMessage.prototype = {
    if (this.flags.indexOf(flag) == -1)
      this.flags.push(flag);
   },
-  clearFlag : function (flag) {
-    if ((index = this.flags.indexOf(flag)) != -1)
-    this.flags.splice(index, 1);
-  },
-   // This allows us to simulate servers that approximate the rfc822 size.
+  // This allows us to simulate servers that approximate the rfc822 size.
   setSize: function(size) {
     this.size = size;
   },
+  clearFlag : function (flag) {
+    if ((index = this.flags.indexOf(flag)) != -1)
+      this.flags.splice(index, 1);
+  },
   getText : function (start, length) {
     if (!start)
       start = 0;
@@ -385,7 +384,7 @@ imapMessage.prototype = {
     } else {
       throw "Can't get subparts!";
     }
-    
+
     if (!gIOService)
       gIOService = Cc["@mozilla.org/network/io-service;1"]
                      .getService(Ci.nsIIOService);
@@ -421,7 +420,7 @@ imapMessage.prototype = {
 
       return headers;
     }
-    
+
     return requestListener.answer;
   }
 }
@@ -434,7 +433,7 @@ const IMAP_FLAG_CASE_INSENSITIVE = 1;
 /**
  * This flag represents whether or not CREATE hierarchies need a delimiter.
  *
- * If this flag is off, <tt>CREATE a<br />CREATE a/b</tt> fails where 
+ * If this flag is off, <tt>CREATE a<br />CREATE a/b</tt> fails where
  * <tt>CREATE a/<br />CREATE a/b</tt> would succeed (assuming the delimiter is
  * '/').
  */
@@ -572,7 +571,7 @@ function formatArg(argument, spec) {
     throw "Expected argument of type " + spec + "!";
 
   if (spec == "atom") {
-    argument = argument.toUpperCase(); 
+    argument = argument.toUpperCase();
   } else if (spec == "mailbox") {
     var converter = Cc["@mozilla.org/intl/scriptableunicodeconverter"]
                       .createInstance(Ci.nsIScriptableUnicodeConverter);
@@ -639,10 +638,28 @@ function formatArg(argument, spec) {
 function IMAP_RFC3501_handler(daemon) {
   this._daemon = daemon;
   this.closing = false;
-  this._state = IMAP_STATE_NOT_AUTHED;
-  this._authenticating = undefined;
+  this.dropOnStartTLS = false;
+  // This can be used to simulate timeouts on large copies
+  this.copySleep = 0;
+  // This can be used to cause the artificial failure of any given command.
+  this.commandToFail = "";
+  // map: property = auth scheme {String}, value = start function on this obj
+  this._kAuthSchemeStartFunction = {};
+
+  this.resetTest();
 }
 IMAP_RFC3501_handler.prototype = {
+
+  kUsername : "user",
+  kPassword : "password",
+  kAuthSchemes : [], // Added by RFC2195 extension. Test may modify as needed.
+  kCapabilities : [/*"LOGINDISABLED", "STARTTLS",*/], // Test may modify as needed.
+
+  resetTest : function() {
+    this._state = IMAP_STATE_NOT_AUTHED;
+    this._multiline = false;
+    this._nextAuthFunction = undefined; // should be in RFC2195_ext, but too lazy
+  },
   onStartup : function () {
     return "* OK IMAP4rev1 Fakeserver started up";
   },
@@ -676,6 +693,9 @@ IMAP_RFC3501_handler.prototype = {
     // If we're here, we have a command with arguments. Dispatch!
     return this._dispatchCommand(command, args);
   },
+  onServerFault: function (e) {
+    return this._tag + " BAD internal server error: " + e;
+  },
   onMultiline : function (line) {
     // A multiline arising form a literal being passed
     if (this._partial) {
@@ -709,36 +729,33 @@ IMAP_RFC3501_handler.prototype = {
       this._multiline = false;
       return this._dispatchCommand(command, args);
     }
-    if (this._authenticating) {
-      line = atob(line);
+
+    if (this._nextAuthFunction) {
+      var func = this._nextAuthFunction;
+      this._multiline = false;
+      this._nextAuthFunction = undefined;
       if (line == "*") {
-        this._authenticating = undefined;
-        this._multiline = false;
-        return this._tag + " BAD okay, I won't authenticate you.";
+        return this._tag + " BAD Okay, as you wish. Chicken";
       }
-      // base64 encoded <nul>user<nul>password
-      if (line == atob("AHVzZXIAcGFzc3dvcmQ=")) {
-        this._authenticating = undefined;
-        this._state = IMAP_STATE_AUTHED;
-        this._multiline = false;
-        return this._tag + " OK I just authenticated you. Happy now?";
-      }
-      else {
-        this._authenticating = undefined;
-        this._multiline = false;
-        return this._tag + " BAD invalid password, I won't authenticate you.";
+      if (!func || typeof(func) != "function") {
+        return this._tag + " BAD I'm lost. Internal server error during auth";
       }
+      try {
+        return this._tag + " " + func.call(this, line);
+      } catch (e) { return this._tag + " BAD " + e; }
     }
     return undefined;
   },
   _dispatchCommand : function (command, args) {
     command = command.toUpperCase();
+    if (command == this.commandToFail.toUpperCase())
+      return this._tag + " NO " + command + " failed";
     if (command in this) {
       this._lastCommand = command;
       // Are we allowed to execute this command?
       if (this._enabledCommands[this._state].indexOf(command) == -1)
-        return this._tag + " BAD illegal command for current state";
-      
+        return this._tag + " BAD illegal command for current state " + this._state;
+
       try {
         // Format the arguments nicely
         args = this._treatArgs(args, command);
@@ -749,7 +766,7 @@ IMAP_RFC3501_handler.prototype = {
         var response = e;
       }
     } else {
-      var response = "BAD parse error: command not implemented";
+      var response = "BAD " + command  + " not implemented";
     }
 
     // Add status updates
@@ -844,7 +861,7 @@ IMAP_RFC3501_handler.prototype = {
     NOOP : [],
     LOGOUT : [],
     STARTTLS : [],
-    AUTHENTICATE : ["atom"],
+    AUTHENTICATE : ["atom", "..."],
     LOGIN : ["string", "string"],
     SELECT : ["mailbox"],
     EXAMINE : ["mailbox"],
@@ -872,30 +889,48 @@ IMAP_RFC3501_handler.prototype = {
   // (ordered as in spec) //
   //////////////////////////
   CAPABILITY : function (args) {
-    return "* CAPABILITY IMAP4rev1 " + this._capabilities.join(" ") + "\0" +
-           "OK CAPABILITY completed";
+    var capa = "* CAPABILITY IMAP4rev1 " + this.kCapabilities.join(" ");
+    if (this.kAuthSchemes.length > 0)
+      capa += " AUTH=" + this.kAuthSchemes.join(" AUTH=");
+    capa += "\0" + "OK CAPABILITY completed";
+    return capa;
   },
-  _capabilities : [/*"LOGINDISABLED", "STARTTLS",*/ "AUTH=PLAIN"],
   LOGOUT : function (args) {
     this.closing = true;
     if (this._selectedMailbox)
       this._daemon.synchronize(this._selectedMailbox, !this._readOnly);
+    this._state = IMAP_STATE_NOT_AUTHED;
     return "* BYE IMAP4rev1 Logging out\0OK LOGOUT completed";
   },
   NOOP : function (args) {
     return "OK NOOP completed";
   },
   STARTTLS : function (args) {
-    return "BAD maild doesn't support TLS ATM";
+    // simulate annoying server that drops connection on STARTTLS
+    if (this.dropOnStartTLS) {
+      this.closing = true;
+      return "";
+    }
+    else
+      return "BAD maild doesn't support TLS ATM";
   },
+  _nextAuthFunction : undefined,
   AUTHENTICATE : function (args) {
-    // TODO: check the args
-    this._authenticating = args;
-    this._multiline = true;
-    return "+";
+    var scheme = args[0]; // already uppercased by type "atom"
+    // |scheme| contained in |kAuthSchemes|?
+    if (!this.kAuthSchemes.some(function (s) { return s == scheme; }))
+      return "-ERR AUTH " + scheme + " not supported";
+
+    var func = this._kAuthSchemeStartFunction[scheme];
+    if (!func || typeof(func) != "function")
+      return "BAD I just pretended to implement AUTH " + scheme + ", but I don't";
+    return func.call(this, args[1]);
   },
   LOGIN : function (args) {
-    if (args == "\"user\" \"password\"") {
+    if (this.kCapabilities.some(function(c) { return c == "LOGINDISABLED"; } ))
+      return "BAD old-style LOGIN is disabled, use AUTHENTICATE";
+    if (args[0] == this.kUsername &&
+        args[1] == this.kPassword) {
       this._state = IMAP_STATE_AUTHED;
       return "OK authenticated";
     }
@@ -907,13 +942,13 @@ IMAP_RFC3501_handler.prototype = {
     var box = this._daemon.getMailbox(args[0]);
     if (!box)
       return "NO no such mailbox";
-    
+
     if (this._selectedMailbox)
       this._daemon.synchronize(this._selectedMailbox, !this._readOnly);
     this._state = IMAP_STATE_SELECTED;
     this._selectedMailbox = box;
     this._readOnly = false;
-    
+
     var response = "* FLAGS (" + box.msgflags.join(" ") + ")\0";
     response += "* " + box._messages.length + " EXISTS\0* ";
     response += box._messages.reduce(function (count, message) {
@@ -970,8 +1005,11 @@ IMAP_RFC3501_handler.prototype = {
     var mbox = this._daemon.getMailbox(args[0]);
     if (!mbox || mbox.name == "")
       return "NO no such mailbox";
-    if (mbox._children.length > 0 && "\\Noselect" in mbox.flags)
-      return "NO cannot delete mailbox";
+    if (mbox._children.length > 0) {
+      for (let i = 0; i < mbox.flags.length; i++)
+        if (mbox.flags[i] == "\\Noselect")
+          return "NO cannot delete mailbox";
+    }
     this._daemon.deleteMailbox(mbox);
     return "OK DELETE completed";
   },
@@ -1024,6 +1062,9 @@ IMAP_RFC3501_handler.prototype = {
     var box = this._daemon.getMailbox(args[0]);
     if (!box)
       return "NO no such mailbox exists";
+    for (let i = 0; i < box.flags.length; i++)
+      if (box.flags[i] == "\\Noselect")
+        return "NO STATUS not allowed on Noselect folder";
     var parts = [];
     for each (var status in args[1]) {
       var line = status + " ";
@@ -1129,7 +1170,7 @@ IMAP_RFC3501_handler.prototype = {
     }
     if (uid && args[1].indexOf("UID") == -1)
       args[1].push("UID");
-    
+
     // Step 2.1: Preprocess the item fetch stack
     var items = [], prefix = undefined;
     for each (var item in args[1]) {
@@ -1171,7 +1212,7 @@ IMAP_RFC3501_handler.prototype = {
         try {
           parts.push(this[functionName](messages[i], item));
         } catch (ex) {
-          
+
           return "BAD error in fetching: "+ex;
         }
       }
@@ -1226,14 +1267,24 @@ IMAP_RFC3501_handler.prototype = {
     var dest = this._daemon.getMailbox(args[1]);
     if (!dest)
       return "NO [TRYCREATE] what mailbox?";
-    
+
     for each (var message in messages) {
       let newMessage = new imapMessage(message._URI, dest.uidnext++,
                                        message.flags);
       newMessage.recent = false;
       dest.addMessage(newMessage);
     }
-
+    if (this.copySleep > 0) {
+      // spin rudely for copyTimeout milliseconds.
+      let now = new Date();
+      let alarm;
+      let startingMSeconds = now.getTime();
+      while (true) {
+        alarm = new Date();
+        if (alarm.getTime() - startingMSeconds > this.copySleep)
+          break;
+      }
+    }
     return "OK COPY completed";
   },
   UID : function (args) {
@@ -1244,14 +1295,16 @@ IMAP_RFC3501_handler.prototype = {
     return this[name](args, true);
   },
 
-  postCommand : function (obj) {
-    if (this.closing)
-      obj.closeSocket();
+  postCommand : function (reader) {
+    if (this.closing) {
+      this.closing = false;
+      reader.closeSocket();
+    }
     if (this.sendingLiteral)
-      obj.preventLFMunge();
-    obj.setMultiline(this._multiline);
-    if (this._lastCommand == obj.watchWord)
-      obj.stopTest();
+      reader.preventLFMunge();
+    reader.setMultiline(this._multiline);
+    if (this._lastCommand == reader.watchWord)
+      reader.stopTest();
   },
   onServerFault : function () {
     return ("_tag" in this ? this._tag : '*') + ' BAD Internal server fault.';
@@ -1294,6 +1347,10 @@ IMAP_RFC3501_handler.prototype = {
         else
           return daemon._selectedMailbox._messages.length;
       }
+      let re = /[0-9]/g;
+      let num = part.match(re);
+      if(!num || (num.length != part.length))
+        throw "BAD invalid UID " + part;
       return parseInt(part);
     }
 
@@ -1320,7 +1377,7 @@ IMAP_RFC3501_handler.prototype = {
       if (set[i] == set[i - 1])
         set.splice(i, 0);
     }
-  
+
     if (!ids)
       ids = [];
     if (uid) {
@@ -1349,7 +1406,7 @@ IMAP_RFC3501_handler.prototype = {
 
     if (parts[0] != "BODY.PEEK" && !this._readOnly)
       message.setFlag("\\Seen");
-   
+
     if (parts[3])
       parts[3] = parts[3].split(/\./).map(function (e) { return parseInt(e); });
 
@@ -1391,8 +1448,13 @@ IMAP_RFC3501_handler.prototype = {
     // error here, it will pop until it gets to FETCH, which will just pop at a
     // BAD response, which is what should happen if the query is malformed.
     // Now we dump it all off onto imapMessage to mess with.
-    var information = message.getPart(partNum, false);
-    
+    let information = "";
+    let bodyPart = new createBodyPart(message.getText());
+    if (partNum == "")
+      information = message.getPart(partNum, false);
+    else
+      bodyPart.partNum = partNum.split(/\./);
+
     // Start off the response
     var response = "BODY[" + parts[1] + "]";
     if (parts[3])
@@ -1406,9 +1468,14 @@ IMAP_RFC3501_handler.prototype = {
     switch (query) {
     case "":
     case "TEXT":
-    case "HEADER":
+      data +=  bodyPart.bodyText;
+      break;
+    case "HEADER": // I believe this specifies mime for an RFC822 message only
+      data += bodyPart.mime + "\r\n\r\n";
+      break;
     case "MIME":
-      throw "Not yet supported!";
+      data += bodyPart.mime + "\r\n\r\n";
+      break;
     case "HEADER.FIELDS":
       var joinList = [];
       /*for each (let header in queryArgs) {
@@ -1438,18 +1505,22 @@ IMAP_RFC3501_handler.prototype = {
           wantFold = true;
         }
       }
-      data = joinList.join('\r\n');
+      data = joinList.join('\r\n') + "\r\n";
       break;
     case "HEADER.FIELDS.NOT":
+      data += partNum + query + " not yet supported\r\n";
+      break;
     default:
-      throw "Can't do BODY[" + query + "]";
+      data += bodyPart.bodyText;
     }
 
     response += '{' + data.length + '}\r\n';
     response += data;
     return response;
   },
-  //_FETCH_BODYSTRUCTURE,
+  _FETCH_BODYSTRUCTURE : function (message, query) {
+    return "BODYSTRUCTURE " + bodystructure(message.getText());
+  },
   //_FETCH_ENVELOPE,
   _FETCH_FLAGS : function (message) {
     var response = "FLAGS (";
@@ -1474,7 +1545,7 @@ IMAP_RFC3501_handler.prototype = {
     if (query == "RFC822.TEXT")
       return this._FETCH_BODY(message, "BODY[TEXT]")
                  .replace("BODY[TEXT]", "RFC822.TEXT");
-    
+
     if (query == "RFC822.SIZE") {
       var channel = message.channel;
       var length = message.size ? message.size : channel.contentLength;
@@ -1509,12 +1580,12 @@ IMAP_RFC3501_handler.prototype = {
 ////////////////////////////////////////////////////////////////////////////////
 
 var configurations = {
-  Cyrus: ["RFC2342"],
-  UW: ["RFC2342"],
-  Dovecot: [],
-  Zimbra: ["RFC2342"],
-  Exchange: ["RFC2342"],
-  LEMONADE: ["RFC2342"]
+  Cyrus: ["RFC2342", "RFC2195"],
+  UW: ["RFC2342", "RFC2195"],
+  Dovecot: ["RFC2195"],
+  Zimbra: ["RFC2342", "RFC2195"],
+  Exchange: ["RFC2342", "RFC2195"],
+  LEMONADE: ["RFC2342", "RFC2195"],
 };
 
 function mixinExtension(handler, extension) {
@@ -1568,7 +1639,7 @@ var IMAP_RFC2342_extension = {
     }
     return response;
   },
-  _capabilities : ["NAMESPACE"],
+  kCapabilities : ["NAMESPACE"],
   _argFormat : { NAMESPACE : [] },
   // Enabled in AUTHED and SELECTED states
   _enabledCommands : { 1 : ["NAMESPACE"], 2 : ["NAMESPACE"] }
@@ -1606,5 +1677,266 @@ var IMAP_RFC4315_extension = {
     }
     return response;
   },
-  _capabilities: ["UIDPLUS"]
+  kCapabilities: ["UIDPLUS"]
 };
+
+
+/**
+ * This implements AUTH schemes. Could be moved into RFC3501 actually.
+ * The test can en-/disable auth schemes by modifying kAuthSchemes.
+ */
+var IMAP_RFC2195_extension = {
+  kAuthSchemes : [ "CRAM-MD5" , "PLAIN", "LOGIN" ],
+
+  preload: function (handler) {
+    handler._kAuthSchemeStartFunction["CRAM-MD5"] = this.authCRAMStart;
+    handler._kAuthSchemeStartFunction["PLAIN"] = this.authPLAINStart;
+    handler._kAuthSchemeStartFunction["LOGIN"] = this.authLOGINStart;
+  },
+
+  authPLAINStart : function (lineRest)
+  {
+    this._nextAuthFunction = this.authPLAINCred;
+    this._multiline = true;
+
+    return "+";
+  },
+  authPLAINCred : function (line)
+  {
+    var req = AuthPLAIN.decodeLine(line);
+    if (req.username == this.kUsername &&
+        req.password == this.kPassword) {
+      this._state = IMAP_STATE_AUTHED;
+      return "OK Hello friend! Friends give friends good advice: Next time, use CRAM-MD5";
+    }
+    else {
+      return "BAD Wrong username or password, crook!";
+    }
+  },
+
+  authCRAMStart : function (lineRest)
+  {
+    this._nextAuthFunction = this.authCRAMDigest;
+    this._multiline = true;
+
+    this._usedCRAMMD5Challenge = AuthCRAM.createChallenge("localhost");
+    return "+ " + this._usedCRAMMD5Challenge;
+  },
+  authCRAMDigest : function (line)
+  {
+    var req = AuthCRAM.decodeLine(line);
+    var expectedDigest = AuthCRAM.encodeCRAMMD5(
+        this._usedCRAMMD5Challenge, this.kPassword);
+    if (req.username == this.kUsername &&
+        req.digest == expectedDigest) {
+      this._state = IMAP_STATE_AUTHED;
+      return "OK Hello friend!";
+    }
+    else {
+      return "BAD Wrong username or password, crook!";
+    }
+  },
+
+  authLOGINStart : function (lineRest)
+  {
+    this._nextAuthFunction = this.authLOGINUsername;
+    this._multiline = true;
+
+    return "+ " + btoa("Username:");
+  },
+  authLOGINUsername : function (line)
+  {
+    var req = AuthLOGIN.decodeLine(line);
+    if (req == this.kUsername)
+      this._nextAuthFunction = this.authLOGINPassword;
+    else // Don't return error yet, to not reveal valid usernames
+      this._nextAuthFunction = this.authLOGINBadUsername;
+    this._multiline = true;
+    return "+ " + btoa("Password:");
+  },
+  authLOGINBadUsername : function (line)
+  {
+    return "BAD Wrong username or password, crook!";
+  },
+  authLOGINPassword : function (line)
+  {
+    var req = AuthLOGIN.decodeLine(line);
+    if (req == this.kPassword) {
+      this._state = IMAP_STATE_AUTHED;
+      return "OK Hello friend! Where did you pull out this old auth scheme?";
+    }
+    else {
+      return "BAD Wrong username or password, crook!";
+    }
+  },
+};
+
+// FETCH BODY PARTS
+// part number is assumed valid
+function createBodyPart(msg, apn) {
+  if (!msg || msg == "") {
+    this.index = null;
+    this.mime = "";
+    this.bodyText = "";
+    return;
+  }
+  this.BndryAttrRE = /boundary="([^"]+)"/img; // g is to continue where left off
+                                              // and enable use of lastIndex
+  this._msg = msg;
+  if (apn) {
+    this._pn  = apn;
+    this.init();
+  }
+}
+
+createBodyPart.prototype = {
+  set msg(x){
+    if (!x || x == "") {
+      this.index = null;
+      this.mime = "";
+      this.bodyText = "";
+      this._msg = "";
+      return;
+    }
+    this._msg = x;
+  },
+
+  // setting the partnum here will re-init if we have a msg
+  set partNum(x){
+    if (!x || x.length == 0)
+      return;
+    this._pn = x;
+    if (this._msg.length > 0)
+      this.init();
+  },
+
+  init : function() {
+    this.BndryAttrRE.lastIndex = 0;
+    for each (let x in this._pn) {
+      var bndryAttrArray = this.BndryAttrRE(this._msg);
+
+      if (!bndryAttrArray)  // FIXME--must have done a BODY[0] of a non-multipart
+        return;             // message. This may not be possible but let it be
+                            // known it needs a fix if this is done in real world
+                            // The RFC specs indicate bodyparts start at 1. Our
+                            // backend code uses bodypart 0 internally only.
+                            // We may want to throw here if we are asking server
+                            // for BODY[0]
+
+      var bndryTag = Array.map("--" + bndryAttrArray[1],
+                               function(a) {
+                                 return '\\x' + parseInt(a.charCodeAt(0),10)
+                                                          .toString(16)})
+                          .join('');// now have a pattern like '\xnn\xnn\xnn\xnn'
+                                    // to allow boundaries with '+' and other
+                                    // regex metacharacters
+      var bndryRE = new RegExp(bndryTag + "([\\s\\S]*?)\\r\\n", "gm");
+      while (x > 0) {
+        bndryRE(this._msg); //need a check for null maybe. nah.
+        --x;
+      }
+    }
+    this.index = bndryRE.lastIndex; // start of mime
+    var mimeArrayRE = /[\s\S]*?\r\n\r\n/g
+    mimeArrayRE.lastIndex = this.index;
+    var mimeArray = mimeArrayRE(this._msg);
+    this.mime = mimeArray[0];
+    this.mime = this.mime.replace(/\r\n\s+/g," ");
+    var textRE;
+
+    // if we have a mime part and it is a multipart find its terminator
+    if (mimeArray && /boundary=/.test(mimeArray[0]))
+      textRE= new RegExp("([\\s\\S]*?)\\r\\n" + bndryTag + "--", "mg");
+    // else just find next boundary
+    else
+      textRE = new RegExp("([\\s\\S]*?)\\r\\n" + bndryTag , "mg");
+
+    textRE.lastIndex = mimeArrayRE.lastIndex;
+    this.bodyText= textRE(this._msg)[1];
+  }
+}
+
+// FETCH BODYSTRUCTURE
+function bodystructure(msg) {
+  if (!msg || msg == "")
+    return "";
+  var res = "";
+  const MimeRE = /([\s\S]*?)\r\n\r\n/gm;
+  MimeRE.lastIndex = 0;
+  const BndryAttrRE = /boundary="([^"]+)"/img; // g is to continue where left off
+                                               // and enable use of lastIndex
+  BndryAttrRE.lastIndex = 0;
+
+  var filterBodyStructure = function(aStr, aBndryTag)
+  {
+    var isTerm;
+    if (!aStr || aStr == "")
+      return;
+
+    var mime = MimeRE(aStr); // mime[1] is mime string
+    var contentType = [];
+    var contentTransferEncoding;
+    if (mime[1]){
+      mime[1] = mime[1].replace(/\r\n\s*/g," ") // folding
+
+      // save mime info
+      // contentType[1] is type and contentType[2] is subtype
+      contentType = /content-type:[^\S]*([^\/]*)\/([^\/;\s]*)/im(mime[1]);
+      contentType[1] = contentType[1].toUpperCase();
+      contentType[2] = contentType[2].toUpperCase();
+      contentTransferEncoding = /Content-Transfer-Encoding:[^\S]*([^;\s]*)/im(mime[1]);
+      contentTransferEncoding = contentTransferEncoding ?
+                                contentTransferEncoding[1].toUpperCase() :
+                                "7BIT";
+    } else {  //default to plain/text
+      contentType[1] = "TEXT";
+      contentType[2] = "PLAIN";
+      contentTransferEncoding = "7BIT";
+    }
+    if (contentType[1] == "MULTIPART") {
+      res += "(";
+
+      // get boundary. Local scope lastIndex
+      BndryAttrRE.lastIndex = 0;
+      var bndryAttrArray = BndryAttrRE(mime[1]);
+      var bndryTag = Array.map("--" + bndryAttrArray[1],
+                           function(a) {
+                             return '\\x' + parseInt(a.charCodeAt(0),10).toString(16)})
+                          .join('');
+      var bndryRE = new RegExp(bndryTag + "(..)?", "gm");
+
+      // goto boundary
+      bndryRE.lastIndex = MimeRE.lastIndex;
+      // find boundary tag and check if terminated)
+      isTerm = bndryRE(aStr)[1] == "--";
+
+      // loop to get all parts
+      while(MimeRE.lastIndex > 0 && !isTerm) {
+        MimeRE.lastIndex = bndryRE.lastIndex;
+        filterBodyStructure(aStr, bndryTag)
+        isTerm = bndryRE(aStr)[1] == "--";
+      }
+
+      // write mime info
+      res += ' "'+contentType[2]+'" ("BOUNDARY" "'+bndryAttrArray[1]+'") NIL NIL)';
+    } else {
+      var tmpRE = new RegExp("([\\s\\S]*?)\\r\\n" + aBndryTag, "gm");
+      tmpRE.lastIndex = MimeRE.lastIndex;
+      var tmpArr = tmpRE(aStr);
+      var lines = 0;
+      var len;
+      if (tmpArr) {
+        len = tmpArr[1].length;
+        for each (let i in tmpArr[1]) {
+          lines += (i == '\r') ? 1 : 0 ;
+        }
+      }
+      res += '("' + contentType[1] + '" "' + contentType[2] +
+             '" NIL NIL NIL "' +  contentTransferEncoding + '" ' + len +
+             ((contentType[1] == "TEXT") ? (" " + lines) : "") +
+             ' NIL NIL NIL)';
+    }
+  };
+  filterBodyStructure(msg);
+  return res;
+}
