@@ -15,13 +15,14 @@
  * The Original Code is Mozilla Foundation code.
  *
  * The Initial Developer of the Original Code is Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2005
+ * Portions created by the Initial Developer are Copyright (C) 2005-2009
  * the Initial Developer. All Rights Reserved.
  *
  * Contributor(s):
  *   Stuart Parmenter <stuart@mozilla.com>
  *   Masayuki Nakano <masayuki@d-toybox.com>
  *   John Daggett <jdaggett@mozilla.com>
+ *   Jonathan Kew <jfkthame@gmail.com>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -51,6 +52,7 @@
 #include "gfxSkipChars.h"
 #include "gfxRect.h"
 #include "nsExpirationTracker.h"
+#include "gfxFontConstants.h"
 
 #ifdef DEBUG
 #include <stdio.h>
@@ -60,23 +62,26 @@ class gfxContext;
 class gfxTextRun;
 class nsIAtom;
 class gfxFont;
+class gfxFontFamily;
 class gfxFontGroup;
 class gfxUserFontSet;
 class gfxUserFontData;
 
-#define FONT_STYLE_NORMAL              0
-#define FONT_STYLE_ITALIC              1
-#define FONT_STYLE_OBLIQUE             2
+// We should eliminate these synonyms when it won't cause many merge conflicts.
+#define FONT_STYLE_NORMAL              NS_FONT_STYLE_NORMAL
+#define FONT_STYLE_ITALIC              NS_FONT_STYLE_ITALIC
+#define FONT_STYLE_OBLIQUE             NS_FONT_STYLE_OBLIQUE
 
-#define FONT_WEIGHT_NORMAL             400
-#define FONT_WEIGHT_BOLD               700
+// We should eliminate these synonyms when it won't cause many merge conflicts.
+#define FONT_WEIGHT_NORMAL             NS_FONT_WEIGHT_NORMAL
+#define FONT_WEIGHT_BOLD               NS_FONT_WEIGHT_BOLD
 
 #define FONT_MAX_SIZE                  2000.0
 
 struct THEBES_API gfxFontStyle {
     gfxFontStyle();
-    gfxFontStyle(PRUint8 aStyle, PRUint16 aWeight, gfxFloat aSize,
-                 const nsACString& aLangGroup,
+    gfxFontStyle(PRUint8 aStyle, PRUint16 aWeight, PRInt16 aStretch,
+                 gfxFloat aSize, const nsACString& aLangGroup,
                  float aSizeAdjust, PRPackedBool aSystemFont,
                  PRPackedBool aFamilyNameQuirks,
                  PRPackedBool aPrinterFont);
@@ -104,6 +109,10 @@ struct THEBES_API gfxFontStyle {
     // font being used, since it is two weights lighter than 900.
     PRUint16 weight;
 
+    // The stretch of the font (the sum of various NS_FONT_STRETCH_*
+    // constants; see gfxFontConstants.h).
+    PRInt16 stretch;
+
     // The logical size of the font, in pixels
     gfxFloat size;
 
@@ -140,6 +149,7 @@ struct THEBES_API gfxFontStyle {
             (printerFont == other.printerFont) &&
             (familyNameQuirks == other.familyNameQuirks) &&
             (weight == other.weight) &&
+            (stretch == other.stretch) &&
             (langGroup.Equals(other.langGroup)) &&
             (sizeAdjust == other.sizeAdjust);
     }
@@ -149,20 +159,26 @@ class gfxFontEntry {
 public:
     THEBES_INLINE_DECL_REFCOUNTING(gfxFontEntry)
 
-    gfxFontEntry(const nsAString& aName) : 
+    gfxFontEntry(const nsAString& aName, gfxFontFamily *aFamily = nsnull,
+                 PRBool aIsStandardFace = PR_FALSE) : 
         mName(aName), mItalic(PR_FALSE), mFixedPitch(PR_FALSE),
         mIsProxy(PR_FALSE), mIsValid(PR_TRUE), 
-        mIsBadUnderlineFont(PR_FALSE),
-        mWeight(500), mStretch(0),
-        mCmapInitialized(PR_FALSE), mUserFontData(nsnull)
+        mIsBadUnderlineFont(PR_FALSE), mIsUserFont(PR_FALSE),
+        mStandardFace(aIsStandardFace),
+        mWeight(500), mStretch(NS_FONT_STRETCH_NORMAL),
+        mCmapInitialized(PR_FALSE), mUserFontData(nsnull),
+        mFamily(aFamily)
     { }
 
     gfxFontEntry(const gfxFontEntry& aEntry) : 
         mName(aEntry.mName), mItalic(aEntry.mItalic), 
         mFixedPitch(aEntry.mFixedPitch), mIsProxy(aEntry.mIsProxy), 
         mIsValid(aEntry.mIsValid), mIsBadUnderlineFont(aEntry.mIsBadUnderlineFont),
+        mIsUserFont(aEntry.mIsUserFont),
+        mStandardFace(aEntry.mStandardFace),
         mWeight(aEntry.mWeight), mCmapInitialized(aEntry.mCmapInitialized),
-        mCharacterMap(aEntry.mCharacterMap), mUserFontData(aEntry.mUserFontData)
+        mCharacterMap(aEntry.mCharacterMap), mUserFontData(aEntry.mUserFontData),
+        mFamily(aEntry.mFamily)
     { }
 
     virtual ~gfxFontEntry();
@@ -170,8 +186,10 @@ public:
     // unique name for the face, *not* the family
     const nsString& Name() const { return mName; }
 
-    PRInt32 Weight() { return mWeight; }
+    PRUint16 Weight() { return mWeight; }
+    PRInt16 Stretch() { return mStretch; }
 
+    PRBool IsUserFont() { return mIsUserFont; }
     PRBool IsFixedPitch() { return mFixedPitch; }
     PRBool IsItalic() { return mItalic; }
     PRBool IsBold() { return mWeight >= 600; } // bold == weights 600 and above
@@ -184,51 +202,170 @@ public:
     }
 
     virtual PRBool TestCharacterMap(PRUint32 aCh);
-    virtual nsresult ReadCMAP() { return 0; }
+    virtual nsresult ReadCMAP();
+
+    const nsString& FamilyName();
+
+    gfxFontFamily    *Family() { return mFamily; }
 
     nsString         mName;
 
     PRPackedBool     mItalic      : 1;
     PRPackedBool     mFixedPitch  : 1;
-
     PRPackedBool     mIsProxy     : 1;
     PRPackedBool     mIsValid     : 1;
-
     PRPackedBool     mIsBadUnderlineFont : 1;
+    PRPackedBool     mIsUserFont  : 1;
+    PRPackedBool     mStandardFace : 1;
 
     PRUint16         mWeight;
-    PRUint16         mStretch;
+    PRInt16          mStretch;
 
     PRPackedBool     mCmapInitialized;
     gfxSparseBitSet  mCharacterMap;
     gfxUserFontData* mUserFontData;
+
+protected:
+    friend class gfxPlatformFontList;
+    friend class gfxMacPlatformFontList;
+    friend class gfxFcFontEntry;
+    friend class gfxFontFamily;
+
+    gfxFontEntry() :
+        mItalic(PR_FALSE), mFixedPitch(PR_FALSE),
+        mIsProxy(PR_FALSE), mIsValid(PR_TRUE), 
+        mIsBadUnderlineFont(PR_FALSE),
+        mIsUserFont(PR_FALSE),
+        mStandardFace(PR_FALSE),
+        mWeight(500), mStretch(NS_FONT_STRETCH_NORMAL),
+        mCmapInitialized(PR_FALSE),
+        mUserFontData(nsnull),
+        mFamily(nsnull)
+    { }
+
+    virtual nsresult GetFontTable(PRUint32 aTableTag, nsTArray<PRUint8>& aBuffer) {
+        return NS_ERROR_FAILURE; // all platform subclasses should reimplement this!
+    }
+
+    gfxFontFamily *mFamily;
 };
 
 
+// used when picking fallback font
+struct FontSearch {
+    FontSearch(const PRUint32 aCharacter, gfxFont *aFont) :
+        mCh(aCharacter), mFontToMatch(aFont), mMatchRank(0) {
+    }
+    const PRUint32         mCh;
+    gfxFont*               mFontToMatch;
+    PRInt32                mMatchRank;
+    nsRefPtr<gfxFontEntry> mBestMatch;
+};
+
+// helper class for adding other family names back into font cache
+class AddOtherFamilyNameFunctor;
+
 class gfxFontFamily {
 public:
     THEBES_INLINE_DECL_REFCOUNTING(gfxFontFamily)
 
     gfxFontFamily(const nsAString& aName) :
-        mName(aName) { }
+        mName(aName), mOtherFamilyNamesInitialized(PR_FALSE), mHasOtherFamilyNames(PR_FALSE),
+        mIsSimpleFamily(PR_FALSE),
+        mHasStyles(PR_FALSE)
+        { }
 
     virtual ~gfxFontFamily() { }
 
     const nsString& Name() { return mName; }
 
+    virtual void LocalizedName(nsAString& aLocalizedName);
+    virtual PRBool HasOtherFamilyNames();
+    
+    nsTArray<nsRefPtr<gfxFontEntry> >& GetFontList() { return mAvailableFonts; }
+    
+    void AddFontEntry(nsRefPtr<gfxFontEntry> aFontEntry) {
+        mAvailableFonts.AppendElement(aFontEntry);
+    }
+
+    // note that the styles for this family have been added
+    void SetHasStyles(PRBool aHasStyles) { mHasStyles = aHasStyles; }
+
     // choose a specific face to match a style using CSS font matching
     // rules (weight matching occurs here)
+    // may return a face that doesn't precisely match (e.g. normal face when no italic face exists)
+    // aNeedsBold is set to true when bolder face couldn't be found, false otherwise
     gfxFontEntry *FindFontForStyle(const gfxFontStyle& aFontStyle, 
                                    PRBool& aNeedsBold);
 
+    // iterates over faces looking for a match with a given characters
+    // used as part of the font fallback process
+    void FindFontForChar(FontSearch *aMatchData);
+
+    // read in other family names, if any, and use functor to add each into cache
+    virtual void ReadOtherFamilyNames(AddOtherFamilyNameFunctor& aOtherFamilyFunctor);
+
+    // find faces belonging to this family (temporary, for Windows FontFamily to override)
+    virtual void FindStyleVariations() { }
+
+    // search for a specific face using the Postscript name
+    gfxFontEntry* FindFont(const nsAString& aPostscriptName);
+
+    // read in cmaps for all the faces
+    void ReadCMAP() {
+        PRUint32 i, numFonts = mAvailableFonts.Length();
+        // called from RunLoader BEFORE CheckForSimpleFamily so that there cannot
+        // be any NULL entries in mAvailableFonts
+        for (i = 0; i < numFonts; i++)
+            mAvailableFonts[i]->ReadCMAP();
+    }
+
+    // set whether this font family is in "bad" underline offset blacklist.
+    void SetBadUnderlineFont(PRBool aIsBadUnderlineFont) {
+        PRUint32 i, numFonts = mAvailableFonts.Length();
+        // this is only used when initially setting up the family,
+        // so CheckForSimpleFamily has not yet been called and there cannot
+        // be any NULL entries in mAvailableFonts
+        for (i = 0; i < numFonts; i++)
+            mAvailableFonts[i]->mIsBadUnderlineFont = aIsBadUnderlineFont;
+    }
+
+    // sort available fonts to put preferred (standard) faces towards the end
+    void SortAvailableFonts();
+
+    // check whether the family fits into the simple 4-face model,
+    // so we can use simplified style-matching;
+    // if so set the mIsSimpleFamily flag (defaults to False before we've checked)
+    void CheckForSimpleFamily();
+
 protected:
-    // fills in an array with weights of faces that match style, returns
-   // number of weights in array
-    virtual PRBool FindWeightsForStyle(gfxFontEntry* aFontsForWeights[], 
-                                       const gfxFontStyle& aFontStyle) 
-    { return PR_FALSE; }
+    // fills in an array with weights of faces that match style,
+    // returns whether any matching entries found
+    virtual PRBool FindWeightsForStyle(gfxFontEntry* aFontsForWeights[],
+                                       PRBool anItalic, PRInt16 aStretch);
+
+    PRBool ReadOtherFamilyNamesForFace(AddOtherFamilyNameFunctor& aOtherFamilyFunctor,
+                                       gfxFontEntry *aFontEntry,
+                                       PRBool useFullName = PR_FALSE);
 
     nsString mName;
+    nsTArray<nsRefPtr<gfxFontEntry> >  mAvailableFonts;
+    PRPackedBool mOtherFamilyNamesInitialized;
+    PRPackedBool mHasOtherFamilyNames;
+    PRPackedBool mIsSimpleFamily;
+    PRPackedBool mHasStyles;
+
+    enum {
+        // for "simple" families, the faces are stored in mAvailableFonts
+        // with fixed positions:
+        kRegularFaceIndex    = 0,
+        kBoldFaceIndex       = 1,
+        kItalicFaceIndex     = 2,
+        kBoldItalicFaceIndex = 3,
+        // mask values for selecting face with bold and/or italic attributes
+        kBoldMask   = 0x01,
+        kItalicMask = 0x02
+    };
 };
 
 struct gfxTextRange {
@@ -499,6 +636,33 @@ protected:
 public:
     virtual ~gfxFont();
 
+    // options for the kind of bounding box to return from measurement
+    typedef enum {
+        LOOSE_INK_EXTENTS,
+            // A box that encloses all the painted pixels, and may
+            // include sidebearings and/or additional ascent/descent
+            // within the glyph cell even if the ink is smaller.
+        TIGHT_INK_EXTENTS,
+            // A box that tightly encloses all the painted pixels
+            // (although actually on Windows, at least, it may be
+            // slightly larger than strictly necessary because
+            // we can't get precise extents with ClearType).
+        TIGHT_HINTED_OUTLINE_EXTENTS
+            // A box that tightly encloses the glyph outline,
+            // ignoring possible antialiasing pixels that extend
+            // beyond this.
+            // NOTE: The default implementation of gfxFont::Measure(),
+            // which works with the glyph extents cache, does not
+            // differentiate between this and TIGHT_INK_EXTENTS.
+            // Whether the distinction is important depends on the
+            // antialiasing behavior of the platform; currently the
+            // distinction is only implemented in the gfxWindowsFont
+            // subclass, because of ClearType's tendency to paint
+            // outside the hinted outline.
+            // Also NOTE: it is relatively expensive to request this,
+            // as it does not use cached glyph extents in the font.
+    } BoundingBoxType;
+
     const nsString& GetName() const { return mFontEntry->Name(); }
     const gfxFontStyle *GetStyle() const { return &mStyle; }
 
@@ -513,7 +677,6 @@ public:
         gfxFloat strikeoutOffset;
         gfxFloat underlineSize;
         gfxFloat underlineOffset;
-        gfxFloat height;
 
         gfxFloat internalLeading;
         gfxFloat externalLeading;
@@ -568,7 +731,7 @@ public:
         gfxFloat mDescent; // always non-negative
         
         // Bounding box that is guaranteed to include everything drawn.
-        // If aTightBoundingBox was set to true when these metrics were
+        // If a tight boundingBox was requested when these metrics were
         // generated, this will tightly wrap the glyphs, otherwise it is
         // "loose" and may be larger than the true bounding box.
         // Coordinates are relative to the baseline left origin, so typically
@@ -625,7 +788,7 @@ public:
      */
     virtual RunMetrics Measure(gfxTextRun *aTextRun,
                                PRUint32 aStart, PRUint32 aEnd,
-                               PRBool aTightBoundingBox,
+                               BoundingBoxType aBoundingBoxType,
                                gfxContext *aContextForTightBoundingBox,
                                Spacing *aSpacing);
     /**
@@ -710,25 +873,16 @@ public:
          * on the spacing provider.
          */
         TEXT_ENABLE_SPACING          = 0x0008,
-        /**
-         * When set, GetSpacing can return negative spacing.
-         */
-        TEXT_ENABLE_NEGATIVE_SPACING = 0x0010,
         /**
          * When set, GetHyphenationBreaks may return true for some character
          * positions, otherwise it will always return false for all characters.
          */
-        TEXT_ENABLE_HYPHEN_BREAKS    = 0x0040,
+        TEXT_ENABLE_HYPHEN_BREAKS    = 0x0010,
         /**
          * When set, the text has no characters above 255 and it is stored
          * in the textrun in 8-bit format.
          */
-        TEXT_IS_8BIT                 = 0x0080,
-        /**
-         * When set, the text may have UTF16 surrogate pairs, otherwise it
-         * doesn't.
-         */
-        TEXT_HAS_SURROGATES          = 0x0100,
+        TEXT_IS_8BIT                 = 0x0020,
         /**
          * When set, the RunMetrics::mBoundingBox field will be initialized
          * properly based on glyph extents, in particular, glyph extents that
@@ -736,18 +890,18 @@ public:
          * and advance width of the glyph). When not set, it may just be the
          * standard font-box even if glyphs overflow.
          */
-        TEXT_NEED_BOUNDING_BOX       = 0x0200,
+        TEXT_NEED_BOUNDING_BOX       = 0x0040,
         /**
          * When set, optional ligatures are disabled. Ligatures that are
          * required for legible text should still be enabled.
          */
-        TEXT_DISABLE_OPTIONAL_LIGATURES = 0x0400,
+        TEXT_DISABLE_OPTIONAL_LIGATURES = 0x0080,
         /**
          * When set, the textrun should favour speed of construction over
          * quality. This may involve disabling ligatures and/or kerning or
          * other effects.
          */
-        TEXT_OPTIMIZE_SPEED          = 0x0800
+        TEXT_OPTIMIZE_SPEED          = 0x0100
     };
 
     /**
@@ -830,7 +984,7 @@ public:
     PRBool CanBreakLineBefore(PRUint32 aPos) {
         NS_ASSERTION(0 <= aPos && aPos < mCharacterCount, "aPos out of range");
         return mCharacterGlyphs[aPos].CanBreakBefore();
-    }    
+    }
 
     PRUint32 GetLength() { return mCharacterCount; }
 
@@ -942,10 +1096,10 @@ public:
     /**
      * Computes the ReflowMetrics for a substring.
      * Uses GetSpacing from aBreakProvider.
-     * @param aTightBoundingBox if true, we make the bounding box tight
+     * @param aBoundingBoxType which kind of bounding box (loose/tight)
      */
     Metrics MeasureText(PRUint32 aStart, PRUint32 aLength,
-                        PRBool aTightBoundingBox,
+                        gfxFont::BoundingBoxType aBoundingBoxType,
                         gfxContext *aRefContextForTightBoundingBox,
                         PropertyProvider *aProvider);
 
@@ -1024,9 +1178,9 @@ public:
      * Trimmed spaces are still counted in the "characters fit" result.
      * @param aMetrics if non-null, we fill this in for the returned substring.
      * If a hyphenation break was used, the hyphen is NOT included in the returned metrics.
-     * @param aTightBoundingBox if true, we make the bounding box in aMetrics tight
+     * @param aBoundingBoxType whether to make the bounding box in aMetrics tight
      * @param aRefContextForTightBoundingBox a reference context to get the
-     * tight bounding box, if aTightBoundingBox is true
+     * tight bounding box, if requested
      * @param aUsedHyphenation if non-null, records if we selected a hyphenation break
      * @param aLastBreak if non-null and result is aMaxLength, we set this to
      * the maximal N such that
@@ -1051,7 +1205,8 @@ public:
                                  PropertyProvider *aProvider,
                                  PRBool aSuppressInitialBreak,
                                  gfxFloat *aTrimWhitespace,
-                                 Metrics *aMetrics, PRBool aTightBoundingBox,
+                                 Metrics *aMetrics,
+                                 gfxFont::BoundingBoxType aBoundingBoxType,
                                  gfxContext *aRefContextForTightBoundingBox,
                                  PRBool *aUsedHyphenation,
                                  PRUint32 *aLastBreak,
@@ -1160,7 +1315,6 @@ public:
             FLAG_NOT_MISSING              = 0x01,
             FLAG_NOT_CLUSTER_START        = 0x02,
             FLAG_NOT_LIGATURE_GROUP_START = 0x04,
-            FLAG_LOW_SURROGATE            = 0x08,
             
             GLYPH_COUNT_MASK = 0x00FFFF00U,
             GLYPH_COUNT_SHIFT = 8
@@ -1186,15 +1340,17 @@ public:
         PRUint32 GetSimpleGlyph() const { return mValue & GLYPH_MASK; }
 
         PRBool IsMissing() const { return (mValue & (FLAG_NOT_MISSING|FLAG_IS_SIMPLE_GLYPH)) == 0; }
-        PRBool IsLowSurrogate() const {
-            return (mValue & (FLAG_LOW_SURROGATE|FLAG_IS_SIMPLE_GLYPH)) == FLAG_LOW_SURROGATE;
-        }
         PRBool IsClusterStart() const {
             return (mValue & FLAG_IS_SIMPLE_GLYPH) || !(mValue & FLAG_NOT_CLUSTER_START);
         }
         PRBool IsLigatureGroupStart() const {
             return (mValue & FLAG_IS_SIMPLE_GLYPH) || !(mValue & FLAG_NOT_LIGATURE_GROUP_START);
         }
+        PRBool IsLigatureContinuation() const {
+            return (mValue & FLAG_IS_SIMPLE_GLYPH) == 0 &&
+                (mValue & (FLAG_NOT_LIGATURE_GROUP_START | FLAG_NOT_MISSING)) ==
+                    (FLAG_NOT_LIGATURE_GROUP_START | FLAG_NOT_MISSING);
+        }
 
         PRBool CanBreakBefore() const { return (mValue & FLAG_CAN_BREAK_BEFORE) != 0; }
         // Returns FLAG_CAN_BREAK_BEFORE if the setting changed, 0 otherwise
@@ -1230,15 +1386,6 @@ public:
                 (aGlyphCount << GLYPH_COUNT_SHIFT);
             return *this;
         }
-        /**
-         * Low surrogates don't have any glyphs and are not the start of
-         * a cluster or ligature group.
-         */
-        CompressedGlyph& SetLowSurrogate() {
-            mValue = (mValue & FLAG_CAN_BREAK_BEFORE) | FLAG_NOT_MISSING |
-                FLAG_LOW_SURROGATE;
-            return *this;
-        }
         PRUint32 GetGlyphCount() const {
             NS_ASSERTION(!IsSimpleGlyph(), "Expected non-simple-glyph");
             return (mValue & GLYPH_COUNT_MASK) >> GLYPH_COUNT_SHIFT;
@@ -1310,11 +1457,6 @@ public:
 
     // API for setting up the textrun glyphs. Should only be called by
     // things that construct textruns.
-    /**
-     * Record every character that is the second half of a surrogate pair.
-     * This should be called after creating a Unicode textrun.
-     */
-    void RecordSurrogates(const PRUnichar *aString);
     /**
      * We've found a run of text that should use a particular font. Call this
      * only during initialization when font substitution has been computed.
@@ -1331,6 +1473,7 @@ public:
     nsresult AddGlyphRun(gfxFont *aFont, PRUint32 aStartCharIndex, PRBool aForceNewRun = PR_FALSE);
     void ResetGlyphRuns() { mGlyphRuns.Clear(); }
     void SortGlyphRuns();
+    void SanitizeGlyphRuns();
 
     // Call the following glyph-setters during initialization or during reshaping
     // only. It is OK to overwrite existing data for a character.
@@ -1345,7 +1488,7 @@ public:
             mCharacterGlyphs[aCharIndex] = aGlyph;
         }
         if (mDetailedGlyphs) {
-            (mDetailedGlyphs.get())[aCharIndex] = nsnull;
+            mDetailedGlyphs[aCharIndex] = nsnull;
         }
     }
     void SetGlyphs(PRUint32 aCharIndex, CompressedGlyph aGlyph,
@@ -1355,7 +1498,7 @@ public:
     
     /**
      * Prefetch all the glyph extents needed to ensure that Measure calls
-     * on this textrun with aTightBoundingBox false will succeed. Note
+     * on this textrun not requesting tight boundingBoxes will succeed. Note
      * that some glyph extents might not be fetched due to OOM or other
      * errors.
      */
@@ -1365,7 +1508,7 @@ public:
     // and gfxFont::GetBoundingBox
     const CompressedGlyph *GetCharacterGlyphs() { return mCharacterGlyphs; }
     const DetailedGlyph *GetDetailedGlyphs(PRUint32 aCharIndex) {
-        return mDetailedGlyphs ? (mDetailedGlyphs.get())[aCharIndex].get() : nsnull;
+        return mDetailedGlyphs ? mDetailedGlyphs[aCharIndex].get() : nsnull;
     }
     PRBool HasDetailedGlyphs() { return mDetailedGlyphs.get() != nsnull; }
     PRUint32 CountMissingGlyphs();
@@ -1465,14 +1608,15 @@ private:
     // result in appunits
     gfxFloat GetPartialLigatureWidth(PRUint32 aStart, PRUint32 aEnd, PropertyProvider *aProvider);
     void AccumulatePartialLigatureMetrics(gfxFont *aFont,
-                                          PRUint32 aStart, PRUint32 aEnd, PRBool aTight,
+                                          PRUint32 aStart, PRUint32 aEnd,
+                                          gfxFont::BoundingBoxType aBoundingBoxType,
                                           gfxContext *aRefContext,
                                           PropertyProvider *aProvider,
                                           Metrics *aMetrics);
 
     // **** measurement helper ****
-    void AccumulateMetricsForRun(gfxFont *aFont, PRUint32 aStart,
-                                 PRUint32 aEnd, PRBool aTight,
+    void AccumulateMetricsForRun(gfxFont *aFont, PRUint32 aStart, PRUint32 aEnd,
+                                 gfxFont::BoundingBoxType aBoundingBoxType,
                                  gfxContext *aRefContext,
                                  PropertyProvider *aProvider,
                                  PRUint32 aSpacingStart, PRUint32 aSpacingEnd,
