@@ -96,8 +96,15 @@ public:
   {
     NS_ASSERTION(mCallback, "Trying to notify about results without a callback!");
 
-    if (mEventStatus->shouldNotify())
+    if (mEventStatus->shouldNotify()) {
+      // Hold a strong reference to the callback while notifying it, so that if
+      // it spins the event loop, the callback won't be released and freed out
+      // from under us.
+      nsCOMPtr<mozIStorageStatementCallback> callback =
+        do_QueryInterface(mCallback);
+
       (void)mCallback->HandleResult(mResults);
+    }
 
     return NS_OK;
   }
@@ -125,8 +132,15 @@ public:
 
   NS_IMETHOD Run()
   {
-    if (mEventStatus->shouldNotify() && mCallback)
+    if (mEventStatus->shouldNotify() && mCallback) {
+      // Hold a strong reference to the callback while notifying it, so that if
+      // it spins the event loop, the callback won't be released and freed out
+      // from under us.
+      nsCOMPtr<mozIStorageStatementCallback> callback =
+        do_QueryInterface(mCallback);
+
       (void)mCallback->HandleError(mErrorObj);
+    }
 
     return NS_OK;
   }
@@ -236,16 +250,7 @@ AsyncExecuteStatements::bindExecuteAndProcessStatement(StatementData &aData,
 {
   mMutex.AssertNotCurrentThreadOwns();
 
-  // Get the statement; this may actually be creating it so deal with the
-  // possibility of error.
-  sqlite3_stmt *stmt;
-  int rc = aData.getSqliteStatement(&stmt);
-  if (rc != SQLITE_OK) {
-    // XXX We really want to get at the database connection so we can call
-    // sqlite3_errmsg.
-    (void)notifyError(rc, "Problem creating async statement.");
-    return false;
-  }
+  sqlite3_stmt *stmt(aData);
   BindingParamsArray *paramsArray(aData);
 
   // Iterate through all of our parameters, bind them, and execute.
@@ -277,21 +282,6 @@ AsyncExecuteStatements::bindExecuteAndProcessStatement(StatementData &aData,
   return continueProcessing;
 }
 
-bool
-AsyncExecuteStatements::executeAndProcessStatement(StatementData &aData,
-                                                   bool aLastStatement)
-{
-  sqlite3_stmt *stmt;
-  int rc = aData.getSqliteStatement(&stmt);
-  if (rc != SQLITE_OK) {
-    // XXX We really want to get at the database connection so we can call
-    // sqlite3_errmsg.
-    (void)notifyError(rc, "Problem creating async statement.");
-    return false;
-  }
-  return executeAndProcessStatement(stmt, aLastStatement);
-}
-
 bool
 AsyncExecuteStatements::executeAndProcessStatement(sqlite3_stmt *aStatement,
                                                    bool aLastStatement)
